package computercomposer.model;

import java.io.File;
import java.util.HashMap;
import jm.JMC;
import jm.music.data.*;
import jm.util.View;
import jm.util.Write;
//import jm.util.Play;

/**
 * This class generates a 2 voices melody
 * @author Pol Girbal i Jornet
 */
public final class Composer implements JMC{
    
    private final Score score;
    private final int numerator;
    private final Part melody;
    private final Part accompaniment;
    private final Tonality tonality;
    private int[] pitches;
    private final Form form;
    private final String[] phrases;
    private final HashMap<String,Phrase> melodyTable;
    private final HashMap<String,Phrase> accompTable;
    
    /**
     * Assigns name, time signature, tempo, instrument, tonality, bars and form
     * @param name The title of the composition
     * @param n The numerator of the time signature
     * @param d The denominator of the time signature
     * @param tempo The tempo of the composition
     * @param inst The instrument of the composition
     * @param p The pitch of the tonality
     * @param m The mode of the tonality
     * @param bars The number of bars for each phrase
     * @param f The form of the composition
     */
    public Composer(String name, int n, int d, int tempo, int inst, int p, int[] m, int bars, String[] f) {
        score = new Score(name);
        numerator = n;
        score.setTimeSignature(n, d);
        score.setTempo(tempo);
        melody = new Part("Melody", inst, 0);
        accompaniment = new Part("Accompaniment", inst, 0);
        tonality = new Tonality(p, m);
        form = new Form(bars);
        phrases = f;
        melodyTable = new HashMap<>();
        accompTable = new HashMap<>();
    }
    
    /**
     * A method that generates and displays a score with a 2 voices melody
     */
    public void compose(){
        for (String ph : phrases) {
            if (melodyTable.containsKey(ph)) {
                Phrase mPhr = new Phrase();
                Phrase aPhr = new Phrase();
                Note[] phrAmelody = melodyTable.get(ph).getNoteArray();
                Note[] phrAaccomp = accompTable.get(ph).getNoteArray();
                for (Note noteR : phrAmelody) {
                    mPhr.addNote(noteR);
                }
                for (Note noteL : phrAaccomp) {
                    aPhr.addNote(noteL);
                }
                melody.addPhrase(mPhr);
                accompaniment.addPhrase(aPhr);
            } else {
                if (ph.equals("B") || ph.equals("C")) {
                    if(tonality.getMode() == MAJOR_SCALE) {
                        tonality.setPitch(tonality.getPitch()-3);
                        tonality.setMode(MINOR_SCALE);
                    } else if(tonality.getMode() == MINOR_SCALE) {
                        tonality.setPitch(tonality.getPitch()+3);
                        tonality.setMode(MAJOR_SCALE);
                    }
                } 
                Phrase mPhr = new Phrase();
                Phrase aPhr = new Phrase();
                int [] estructure = form.getStruct();
                for (int j = 0; j < estructure.length; j++) {
                    pitches = tonality.getChord(estructure[j]);
                    double rA;
                    double sumA = 0;
                    while (sumA < numerator) {
                        if ( j == 0 || j == estructure.length-1 ) {
                            switch (numerator) {
                                case 2:
                                    aPhr.addNote(new Note(pitches[0]-12,HN));
                                    sumA+=2;
                                    break;
                                case 3:
                                    aPhr.addNote(new Note(pitches[0]-12,DHN));
                                    sumA+=3;
                                    break;
                                case 4:
                                    aPhr.addNote(new Note(pitches[0]-12,WN));
                                    sumA+=4;
                                    break;
                                default:
                                    break;
                            }
                        } else {
                            rA = getRhythmAccomp();
                            if (sumA+rA>numerator) rA = numerator - sumA;
                            aPhr.addNote(new Note(getPitch(pitches)-12,rA));
                            sumA+=rA;
                        }
                    }
                    double rM;
                    double sumM = 0;
                    while (sumM < numerator) {
                        if ( j == estructure.length-1 ) {
                            switch (numerator) {
                                case 2:
                                    mPhr.addNote(new Note(pitches[0],HN));
                                    sumM+=2;
                                    break;
                                case 3:
                                    mPhr.addNote(new Note(pitches[0],DHN));
                                    sumM+=3;
                                    break;
                                case 4:
                                    mPhr.addNote(new Note(pitches[0],WN));
                                    sumM+=4;
                                    break;
                                default:
                                    break;
                            }
                        } else {
                            rM = getRhythm();
                            if (sumM + rM > numerator) rM = numerator - sumM;
                            mPhr.addNote(new Note(getPitch(pitches),rM));
                            sumM += rM;
                        }
                    }
                }
                melodyTable.put(ph, mPhr);
                accompTable.put(ph, aPhr);
                melody.addPhrase(mPhr);
                accompaniment.addPhrase(aPhr);
            } 
        }
        melodyTable.clear();
        accompTable.clear();
        score.addPart(melody);
        score.addPart(accompaniment);
        View.notation(score,2,2);
        //Play.midi(score);

    }
    
    /**
     * A method that saves the score as a MIDI file.
     * @param f The name of the file
     */
    public void saveFile(File f) {
        String path = f.getPath();
        if(!path.endsWith(".mid")) path += ".mid";
        Write.midi(score,path);
    }
    
    /**
     * A method that returns a pitch from a given list of pitch values
     * @param pitches The list of pitch values
     * @return a random pitch value
     */
    private int getPitch(int[] pitches) {
        int p = (int) (Math.random()*pitches.length);
        return pitches[p];
    }
    
    /**
     * A method that returns a semi-random rhythm value
     * WN, HN, QN or EN
     * @return rhythm value
     */
    private double getRhythm() {
        Double r = Math.random();
        Double rhythm = 0.0;
        if (r < 0.01) rhythm = WN;
        else if (r >= 0.01 && r < 0.15) rhythm = HN;
        else if (r >= 0.15 && r < 0.6) rhythm = QN;
        else if (r >= 0.6 && r < 1.0) rhythm = EN;
        
        return rhythm;
    }
    
    /**
     * A method taht returns a semi-random rhythm value
     * QN, HN or WN
     * @return rhythm value
     */
    private double getRhythmAccomp() {
        Double r = Math.random();
        Double rhythm;
        if (r < 0.4) rhythm = QN;
        else if (r >= 0.4 && r < 0.7) rhythm = HN;
        else rhythm = WN;
        
        return rhythm;
    }
}